@@ -1,5 +1,5 @@
-// Copyright (C) 2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2011 Conrad Sanderson
+// Copyright (C) 2011-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2011-2012 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -16,9 +16,9 @@
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 arma_inline
-Gen<eT, gen_type>::Gen(const uword in_n_rows, const uword in_n_cols)
+Gen<T1, gen_type>::Gen(const uword in_n_rows, const uword in_n_cols)
   : n_rows(in_n_rows)
   , n_cols(in_n_cols)
   {
@@ -27,20 +27,22 @@ Gen<eT, gen_type>::Gen(const uword in_n_rows, const uword in_n_cols)
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 arma_inline
-Gen<eT, gen_type>::~Gen()
+Gen<T1, gen_type>::~Gen()
   {
   arma_extra_debug_sigprint();
   }
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 arma_inline
-eT
-Gen<eT, gen_type>::generate()
+typename T1::elem_type
+Gen<T1, gen_type>::generate()
   {
+  typedef typename T1::elem_type eT;
+  
        if(is_same_type<gen_type, gen_ones_full>::value == true) { return eT(1);                   }
   else if(is_same_type<gen_type, gen_zeros    >::value == true) { return eT(0);                   }
   else if(is_same_type<gen_type, gen_randu    >::value == true) { return eT(eop_aux_randu<eT>()); }
@@ -50,44 +52,48 @@ Gen<eT, gen_type>::generate()
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 arma_inline
-eT
-Gen<eT, gen_type>::operator[](const uword i) const
+typename T1::elem_type
+Gen<T1, gen_type>::operator[](const uword i) const
   {
+  typedef typename T1::elem_type eT;
+  
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
     return ((i % n_rows) == (i / n_rows)) ? eT(1) : eT(0);
     }
   else
     {
-    return Gen<eT, gen_type>::generate();
+    return Gen<T1, gen_type>::generate();
     }
   }
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 arma_inline
-eT
-Gen<eT, gen_type>::at(const uword row, const uword col) const
+typename T1::elem_type
+Gen<T1, gen_type>::at(const uword row, const uword col) const
   {
+  typedef typename T1::elem_type eT;
+  
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
     return (row == col) ? eT(1) : eT(0);
     }
   else
     {
-    return Gen<eT, gen_type>::generate();
+    return Gen<T1, gen_type>::generate();
     }
   }
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 inline
 void
-Gen<eT, gen_type>::apply(Mat<eT>& out) const
+Gen<T1, gen_type>::apply(Mat<typename T1::elem_type>& out) const
   {
   arma_extra_debug_sigprint();
   
@@ -103,15 +109,17 @@ Gen<eT, gen_type>::apply(Mat<eT>& out) const
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 inline
 void
-Gen<eT, gen_type>::apply_inplace_plus(Mat<eT>& out) const
+Gen<T1, gen_type>::apply_inplace_plus(Mat<typename T1::elem_type>& out) const
   {
   arma_extra_debug_sigprint();
   
   arma_debug_assert_same_size(out.n_rows, out.n_cols, n_rows, n_cols, "addition");
   
+  typedef typename T1::elem_type eT;
+  
   
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
@@ -131,8 +139,8 @@ Gen<eT, gen_type>::apply_inplace_plus(Mat<eT>& out) const
     
     for(i=0, j=1; j<n_elem; i+=2, j+=2)
       {
-      const eT tmp_i = Gen<eT, gen_type>::generate();
-      const eT tmp_j = Gen<eT, gen_type>::generate();
+      const eT tmp_i = Gen<T1, gen_type>::generate();
+      const eT tmp_j = Gen<T1, gen_type>::generate();
       
       out_mem[i] += tmp_i;
       out_mem[j] += tmp_j;
@@ -140,7 +148,7 @@ Gen<eT, gen_type>::apply_inplace_plus(Mat<eT>& out) const
     
     if(i < n_elem)
       {
-      out_mem[i] += Gen<eT, gen_type>::generate();
+      out_mem[i] += Gen<T1, gen_type>::generate();
       }
     }
   
@@ -149,15 +157,17 @@ Gen<eT, gen_type>::apply_inplace_plus(Mat<eT>& out) const
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 inline
 void
-Gen<eT, gen_type>::apply_inplace_minus(Mat<eT>& out) const
+Gen<T1, gen_type>::apply_inplace_minus(Mat<typename T1::elem_type>& out) const
   {
   arma_extra_debug_sigprint();
   
   arma_debug_assert_same_size(out.n_rows, out.n_cols, n_rows, n_cols, "subtraction");
   
+  typedef typename T1::elem_type eT;
+  
   
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
@@ -177,8 +187,8 @@ Gen<eT, gen_type>::apply_inplace_minus(Mat<eT>& out) const
     
     for(i=0, j=1; j<n_elem; i+=2, j+=2)
       {
-      const eT tmp_i = Gen<eT, gen_type>::generate();
-      const eT tmp_j = Gen<eT, gen_type>::generate();
+      const eT tmp_i = Gen<T1, gen_type>::generate();
+      const eT tmp_j = Gen<T1, gen_type>::generate();
       
       out_mem[i] -= tmp_i;
       out_mem[j] -= tmp_j;
@@ -186,7 +196,7 @@ Gen<eT, gen_type>::apply_inplace_minus(Mat<eT>& out) const
     
     if(i < n_elem)
       {
-      out_mem[i] -= Gen<eT, gen_type>::generate();
+      out_mem[i] -= Gen<T1, gen_type>::generate();
       }
     }
   
@@ -195,15 +205,17 @@ Gen<eT, gen_type>::apply_inplace_minus(Mat<eT>& out) const
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 inline
 void
-Gen<eT, gen_type>::apply_inplace_schur(Mat<eT>& out) const
+Gen<T1, gen_type>::apply_inplace_schur(Mat<typename T1::elem_type>& out) const
   {
   arma_extra_debug_sigprint();
   
   arma_debug_assert_same_size(out.n_rows, out.n_cols, n_rows, n_cols, "element-wise multiplication");
   
+  typedef typename T1::elem_type eT;
+  
   
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
@@ -224,8 +236,8 @@ Gen<eT, gen_type>::apply_inplace_schur(Mat<eT>& out) const
     
     for(i=0, j=1; j<n_elem; i+=2, j+=2)
       {
-      const eT tmp_i = Gen<eT, gen_type>::generate();
-      const eT tmp_j = Gen<eT, gen_type>::generate();
+      const eT tmp_i = Gen<T1, gen_type>::generate();
+      const eT tmp_j = Gen<T1, gen_type>::generate();
       
       out_mem[i] *= tmp_i;
       out_mem[j] *= tmp_j;
@@ -233,7 +245,7 @@ Gen<eT, gen_type>::apply_inplace_schur(Mat<eT>& out) const
     
     if(i < n_elem)
       {
-      out_mem[i] *= Gen<eT, gen_type>::generate();
+      out_mem[i] *= Gen<T1, gen_type>::generate();
       }
     }
   
@@ -242,15 +254,17 @@ Gen<eT, gen_type>::apply_inplace_schur(Mat<eT>& out) const
 
 
 
-template<typename eT, typename gen_type>
+template<typename T1, typename gen_type>
 inline
 void
-Gen<eT, gen_type>::apply_inplace_div(Mat<eT>& out) const
+Gen<T1, gen_type>::apply_inplace_div(Mat<typename T1::elem_type>& out) const
   {
   arma_extra_debug_sigprint();
   
   arma_debug_assert_same_size(out.n_rows, out.n_cols, n_rows, n_cols, "element-wise division");
   
+  typedef typename T1::elem_type eT;
+  
   
   if(is_same_type<gen_type, gen_ones_diag>::value == true)
     {
@@ -273,8 +287,8 @@ Gen<eT, gen_type>::apply_inplace_div(Mat<eT>& out) const
     
     for(i=0, j=1; j<n_elem; i+=2, j+=2)
       {
-      const eT tmp_i = Gen<eT, gen_type>::generate();
-      const eT tmp_j = Gen<eT, gen_type>::generate();
+      const eT tmp_i = Gen<T1, gen_type>::generate();
+      const eT tmp_j = Gen<T1, gen_type>::generate();
       
       out_mem[i] /= tmp_i;
       out_mem[j] /= tmp_j;
@@ -282,7 +296,7 @@ Gen<eT, gen_type>::apply_inplace_div(Mat<eT>& out) const
     
     if(i < n_elem)
       {
-      out_mem[i] /= Gen<eT, gen_type>::generate();
+      out_mem[i] /= Gen<T1, gen_type>::generate();
       }
     }
   
@@ -292,3 +306,4 @@ Gen<eT, gen_type>::apply_inplace_div(Mat<eT>& out) const
 
 
 //! @}
+
